#ifndef dm_refine_H
#define dm_refine_H

/* Scale-to-scale mapping and refinement of candidates and candidate lists. */
/* Last edited on 2007-01-02 12:58:54 by stolfi */

#define dm_refine_H_COPYRIGHT \
  "Copyright  2005  by the State University of Campinas (UNICAMP)" \
  " and the Federal Fluminense University (UFF)"

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_rung.h>
#include <dm_pairing.h>
#include <dm_dyn.h>
#include <dm_cand.h>

#include <vec.h>

#include <stdint.h>

/* CANDIDATE REFINEMET */

dm_cand_t dm_refine_cand
  ( dm_cand_t *cdold, 
    dm_seq_t *ap, 
    dm_seq_t *bp, 
    int delta,
    int kappa,
    bool_t shrink,
    dm_step_score_proc_t *step_score,
    dm_dyn_tableau_t *tb 
  );
  /* Finds an optimum pairing between the sequences {ap} and {bp} within
    the band of half-width {delta} around the candidate {cdold}, with
    ends possibly extended by {kappa} steps. The candidate {cdold}
    should contain a pairing between {ap} and {bp}.
    
    If {shrink} is TRUE, allows the candidate to shrink arbitrarily.
    If {shrink} is false, the extremal rungs may move only as allowed
    by {delta} and {kappa}. Uses the tableau {tb} as a work area,
    expanding it as needed. */

dm_cand_vec_t dm_refine_cand_vec
  ( dm_cand_vec_t *cdv,
    dm_seq_t *ap, 
    dm_seq_t *bp,
    int delta,
    int kappa,
    bool_t shrink,
    dm_step_score_proc_t *step_score, 
    dm_dyn_tableau_t *tb, 
    int maxCands,
    double frac
  );
  /* Re-optimizes the candidates {cdv[...]} between two sequences {ap,bp}.
    The parameters {delta,kappa,shrink,step_score,tb} are passed to {dm_cand_refine}
    for each candidate.
    
    Keeps at most {maxCands} candidates, and discards any candidates 
    that are (nearly) contained in worse or equivalent ones.  The 
    parameter {frac} specifies the fraction of rung containment
    that means candidate containment. */

#endif

