/* See {dm_score.h} */
/* Last edited on 2006-12-26 00:08:17 by stolfi */

#define dm_score_C_COPYRIGHT \
  "Copyright  2006  by the State University of Campinas (UNICAMP)" \
  " and the Federal Fluminense University (UFF)"

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_rung.h>
#include <dm_score.h>

#include <argparser.h>

#include <assert.h>
#include <stdio.h>
#include <math.h>

double dm_score_datum_rung(double eql, double dif, dm_datum_t *dx, dm_datum_t *dy)
  { double diff = dm_datum_diffsq(dx, dy);
    double ffid = 1 - diff;
    return ffid*eql + diff*dif;
  }

double dm_score_datum_step
  ( double eql, 
    double dif,
    dm_datum_t *d0x, 
    dm_datum_t *d0y, 
    dm_datum_t *d1x,
    dm_datum_t *d1y
  )
  { /* Compute the part {sc_diff} of score that is due to datum diffs along step: */
    double diff = dm_datum_step_diffsq(d0x, d0y, d1x, d1y);
    double ffid = 1 - diff;
    return ffid*eql + diff*dif;
  }

double dm_score_step_increments(double skp, int kx, int ky)
  { assert((kx >= 0) && (kx <= 1));
    assert((ky >= 0) && (ky <= 1));
    return (kx != ky ? skp : 0.0);
  }

double dm_score_rung(dm_score_args_t *sc, dm_rung_t g, dm_seq_t *xp, dm_seq_t *yp)
  { if (dm_rung_is_none(&g) || (!sc->discrete))
      { return 0; }
    else
      { int x = g.c[0];
        int y = g.c[1];
        dm_datum_t dx = dm_seq_get_datum(xp, x);
        dm_datum_t dy = dm_seq_get_datum(yp, y);
        return dm_score_datum_rung(sc->eql, sc->dif, &dx, &dy);
      }
  }

double dm_score_step(dm_score_args_t *sc, dm_rung_t g, dm_rung_t h, dm_seq_t *xp, dm_seq_t *yp)
  { 
    /* Check whether the ends are defined: */
    bool_t gnone = dm_rung_is_none(&g);
    bool_t hnone = dm_rung_is_none(&h);
    
    /* Get hold of the datums of the final rung {h}: */
    dm_datum_t *dhx = (hnone ? NULL : dm_seq_get_datum_address(xp, h.c[0]));
    dm_datum_t *dhy = (hnone ? NULL : dm_seq_get_datum_address(yp, h.c[1]));
    
    /* Compute the datum similarity term {Sd}: */
    double Sd;
    if (sc->discrete)
      { /* The term {Sd} is the rung score at {h}: */
        Sd = (hnone ? 0 : dm_score_datum_rung(sc->eql, sc->dif, dhx, dhy));
      }
    else
      { /* Get hold of the datums of the initial rung {g}: */
        dm_datum_t *dgx = (gnone ? NULL : dm_seq_get_datum_address(xp, g.c[0]));
        dm_datum_t *dgy = (gnone ? NULL : dm_seq_get_datum_address(yp, g.c[1]));
    
        /* The term {Sd} is the integral of rung score between the two pairs: */
        Sd = (gnone||hnone ? 0 : dm_score_datum_step(sc->eql, sc->dif, dgx, dgy, dhx, dhy));
      }
    
    /* Compute the skip term {Sk} (insertion/deletion penalty): */
    int Sk;
    if (gnone || hnone)
      { Sk = 0; }
    else
      { int kx = h.c[0] - g.c[0];
        int ky = h.c[1] - g.c[1];
        Sk = dm_score_step_increments(sc->skp, kx, ky);
      }
    return Sd + Sk;
  }

dm_score_args_t dm_score_args_parse(argparser_t *pp)
  { dm_score_args_t sc;
    if (argparser_keyword_present_next(pp, "discrete"))
      { sc.discrete = TRUE; }
    else if (argparser_keyword_present_next(pp, "smooth"))
      { sc.discrete = FALSE; }
    else 
      { argparser_error(pp, "should specify \"discrete\" or \"smooth\"");
        sc.discrete = FALSE;
      }
    sc.eql = argparser_get_next_double(pp, 0.0, 100.0); 
    sc.dif = argparser_get_next_double(pp, -100.0, 0.0); 
    sc.skp = argparser_get_next_double(pp, -100.0, 0.0); 
    return sc;
  }
